﻿using PSO2SERVER.Database;
using PSO2SERVER.Models;
using PSO2SERVER.Protocol.Packets;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PSO2SERVER.Party
{
    public class Party
    {
        public enum PartyColor : byte
        {
            Red,
            Green,
            Yellow,
            Blue,
        }

        public string name;
        private List<Client> members;
        private Client host;
        public PartyQuest currentQuest;
        public PartySettingsPacket partySetting;
        public string questname;

        public Party(string name, Client host)
        {
            this.name = name;
            this.host = host;
            members = new List<Client>();
            currentQuest = new PartyQuest();
            partySetting = new PartySettingsPacket();
            questname = string.Empty;
            addClientToParty(host);
        }

        public void addClientToParty(Client c)
        {
            if (members.Count < 1)
            {
                c.SendPacket(new PartyInitPacket(new Character[1] { c.Character }));
            }
            else
            {
                // ???
            }
            members.Add(c);
            c.currentParty = this;
        }

        public void removeClientFromParty(Client c)
        {
            if (!members.Contains(c))
            {
                Logger.WriteWarning("[PTY] Client {0} was trying to be removed from {1}, but he was never in {1}!", c._account.Username, name);
                return;
            }

            members.Remove(c);
            //TODO do stuff like send the "remove from party" packet.
        }

        public bool hasClientInParty(Client c)
        {
            return members.Contains(c);
        }

        public Client getPartyHost()
        {
            return host;
        }

        public int getSize()
        {
            return members.Count;
        }

        public List<Client> getMembers()
        {
            return members;
        }
    }

    [Serializable]
    public unsafe struct PartyInfo
    {
        public fixed byte unk1[0x0C];
        public ObjectHeader party_object;  // Assuming ObjectHeader is another class
        public string name;  // Name of the party 0xE7E8, 0xFF
        public fixed byte unk2[0x09];
        public fixed byte unk3[0x03];
        public uint unk4;  // 32-bit unsigned integer
        public uint invite_time;  // Time when the player was invited
        public uint unk6;  // 32-bit unsigned integer
    }

    [Flags]
    public enum PartyFlags : byte
    {
        /// Is the party only for friends.
        FRIENDS_ONLY = 1 << 0,  // 1
        /// Is the party only for alliance members.
        ALLIANCE_ONLY = 1 << 1, // 2
        /// Limit multiplayer requests from other parties.
        LIMIT_OTHERS = 1 << 2,  // 4
        /// Is the party only for a single run.
        SINGLE_RUN = 1 << 3,    // 8
        /// Is the party actively looking for members.
        OPEN = 1 << 4,          // 16
        /// Is the party voice chat focused.
        VC_FOCUS = 1 << 6,      // 64

        // This flag would represent an invalid state (all bits set).
        _ = byte.MaxValue,        // 255 (all flags set)
    }
}
